@@ -28,6 +28,7 @@
 #include <qptrlist.h>
 
 #include "kjs_binding.h"
+#include <kjs/protect.h>
 
 class QTimer;
 class KHTMLView;
@@ -89,7 +90,7 @@ namespace KJS {
     QGuardedPtr<KHTMLPart> part() const { return m_part; }
     virtual void mark();
     virtual Value get(ExecState *exec, const Identifier &propertyName) const;
-    virtual bool hasProperty(ExecState *exec, const Identifier &propertyName) const;
+    virtual bool hasOwnProperty(ExecState *exec, const Identifier &propertyName) const;
     virtual void put(ExecState *exec, const Identifier &propertyName, const Value &value, int attr = None);
     virtual bool toBoolean(ExecState *exec) const;
     int installTimeout(const UString &handler, int t, bool singleShot);
@@ -124,6 +125,9 @@ namespace KJS {
     // Set the current "event" object
     void setCurrentEvent( DOM::Event *evt );
 
+    // Set a place to put a dialog return value when the window is cleared.
+    void setReturnValueSlot(ValueImp **slot) { m_returnValueSlot = slot; }
+
     QPtrDict<JSEventListener> jsEventListeners;
     QPtrDict<JSUnprotectedEventListener> jsUnprotectedEventListeners;
     virtual const ClassInfo* classInfo() const { return &info; }
@@ -137,17 +141,17 @@ namespace KJS {
            ScrollTo, ScrollX, ScrollY, MoveBy, MoveTo, ResizeBy, ResizeTo, Self, _Window, Top, _Screen,
            Image, Option, Alert, Confirm, Prompt, Open, Print, SetTimeout, ClearTimeout,
            Focus, GetSelection, Blur, Close, SetInterval, ClearInterval, CaptureEvents, 
-           ReleaseEvents, AddEventListener, RemoveEventListener, XMLHttpRequest, XMLSerializer,
+           ReleaseEvents, AddEventListener, RemoveEventListener, XMLHttpRequest, XMLSerializer, DOMParser,
 	   Onabort, Onblur, Onchange, Onclick, Ondblclick, Ondragdrop, Onerror, 
 	   Onfocus, Onkeydown, Onkeypress, Onkeyup, Onload, Onmousedown, Onmousemove,
            Onmouseout, Onmouseover, Onmouseup, OnWindowMouseWheel, Onmove, Onreset, Onresize, Onscroll, Onsearch,
            Onselect, Onsubmit, Onunload,
-           Statusbar, Toolbar, FrameElement };
+           Statusbar, Toolbar, FrameElement, ShowModalDialog };
   protected:
     Value getListener(ExecState *exec, int eventId) const;
     void setListener(ExecState *exec, int eventId, Value func);
   private:
-    void updateLayout() const;
+    void updateLayout(bool ignoreStylesheets = true) const;
 
     QGuardedPtr<KHTMLPart> m_part;
     Screen *screen;
@@ -163,6 +167,7 @@ namespace KJS {
     BarInfo *m_toolbar;
     WindowQObject *winq;
     DOM::Event *m_evt;
+    ValueImp **m_returnValueSlot;
   };
 
   /**
@@ -270,7 +275,7 @@ namespace KJS {
   public:
     Konqueror(KHTMLPart *p) : part(p) { }
     virtual Value get(ExecState *exec, const Identifier &propertyName) const;
-    virtual bool hasProperty(ExecState *exec, const Identifier &p) const;
+    virtual bool hasOwnProperty(ExecState *exec, const Identifier &p) const;
     virtual UString toString(ExecState *exec) const;
     virtual const ClassInfo* classInfo() const { return &info; }
     static const ClassInfo info;
