@@ -61,7 +61,7 @@ namespace khtml
   {
       enum Type { Frame, IFrame, Object };
 
-      ChildFrame() { m_bCompleted = false; m_bPreloaded = false; m_type = Frame; m_bNotify = false; }
+      ChildFrame() { m_bCompleted = false; m_bPreloaded = false; m_type = Frame; m_bNotify = false; m_hasFallbackContent = false; }
 
 #if !APPLE_CHANGES
       ~ChildFrame() { if (m_run) m_run->abort(); }
@@ -85,6 +85,7 @@ namespace khtml
     QStringList m_paramNames;
     QStringList m_paramValues;
     bool m_bNotify;
+    bool m_hasFallbackContent;
   };
 }
 
@@ -128,7 +129,7 @@ public:
     m_settings = new KHTMLSettings(*KHTMLFactory::defaultHTMLSettings());
 #endif
     m_bClearing = false;
-    m_bCleared = false;
+    m_bCleared = true;
     m_zoomFactor = 100;
     m_bDnd = true;
 #if !APPLE_CHANGES
