@@ -1240,6 +1240,7 @@ Request::Request(DocLoader* dl, CachedObject *_object, bool _incremental)
     object->setRequest(this);
     incremental = _incremental;
     m_docLoader = dl;
+    multipart = false;
 }
 
 Request::~Request()
@@ -1664,6 +1665,19 @@ void Loader::slotReceivedResponse(KIO::Job* job, NSURLResponse *response)
     ASSERT(response);
     r->object->setResponse(response);
     r->object->setExpireDate(KWQCacheObjectExpiresTime(r->m_docLoader, response), false);
+    
+    if (r->multipart) {
+        ASSERT(r->object->type() == CachedObject::Image);
+        static_cast<CachedImage *>(r->object)->clear();
+        r->m_buffer = QBuffer();
+        if (r->m_docLoader->part())
+            r->m_docLoader->part()->checkCompleted();
+        
+    } else if (KWQResponseIsMultipart(response)) {
+        r->multipart = true;
+        if (!r->object->type() == CachedObject::Image)
+            static_cast<KIO::TransferJob*>(job)->cancel();
+    }
 }
 
 #endif
@@ -1689,7 +1703,9 @@ void Loader::slotData( KIO::Job*job, const QByteArray &data )
     r->m_buffer.writeBlock( data.data(), data.size() );
 #endif
 
-    if(r->incremental)
+    if (r->multipart)
+        r->object->data( r->m_buffer, true ); // the loader delivers the data in a multipart section all at once, send eof
+    else if(r->incremental)
         r->object->data( r->m_buffer, false );
 }
 
@@ -1705,7 +1721,8 @@ int Loader::numRequests( DocLoader* dl ) const
     QPtrDictIterator<Request> lIt( m_requestsLoading );
     for (; lIt.current(); ++lIt )
         if ( lIt.current()->m_docLoader == dl )
-            res++;
+            if (!lIt.current()->multipart)
+                res++;
 
 #if APPLE_CHANGES
     QPtrListIterator<Request> bdIt( m_requestsBackgroundDecoding );
@@ -2550,6 +2567,7 @@ CachedObjectClient *CachedObjectClientWalker::next()
 
 // --------------------------------------
 
+CachedObjectClient::~CachedObjectClient() { }
 void CachedObjectClient::setPixmap(const QPixmap &, const QRect&, CachedImage *) {}
 void CachedObjectClient::setStyleSheet(const DOM::DOMString &/*url*/, const DOM::DOMString &/*sheet*/) {}
 #ifndef KHTML_NO_XBL
