@@ -5,6 +5,12 @@
 #include <qpixmap.h>
 #include "dom/dom_string.h"
 
+#ifndef KHTML_NO_XBL
+namespace XBL {
+    class XBLDocumentImpl;
+};
+#endif
+
 namespace khtml {
     class CachedObject;
     class CachedImage;
@@ -19,6 +25,8 @@ namespace khtml {
     class CachedObjectClient
     {
     public:
+        virtual ~CachedObjectClient();
+
         // clipped pixmap (if it is not yet completely loaded,
         // size of the complete (finished loading) pixmap
         // rectangle of the part that has been loaded very recently
@@ -26,6 +34,9 @@ namespace khtml {
         // return whether we need manual update
 	virtual void setPixmap(const QPixmap &, const QRect&, CachedImage *);
 	virtual void setStyleSheet(const DOM::DOMString &/*url*/, const DOM::DOMString &/*sheet*/);
+#ifndef KHTML_NO_XBL
+        virtual void setXBLDocument(const DOM::DOMString& url, XBL::XBLDocumentImpl* doc);
+#endif
 	virtual void notifyFinished(CachedObject * /*finishedObj*/);
     };
 };
