@@ -591,7 +591,7 @@ void RenderLineEdit::addSearchResult()
 
 void RenderLineEdit::handleFocusOut()
 {
-    if ( widget() && widget()->edited() ) {
+    if ( widget() && widget()->edited() && element()) {
         element()->onChange();
         widget()->setEdited( false );
     }
@@ -1116,7 +1116,7 @@ void RenderSelect::updateFromElement()
     if ( m_optionsChanged ) {
         if (element()->m_recalcListItems)
             element()->recalcListItems();
-        QMemArray<HTMLGenericFormElementImpl*> listItems = element()->listItems();
+        QMemArray<HTMLElementImpl*> listItems = element()->listItems();
         int listIndex;
 
         if (m_useListBox)
@@ -1171,6 +1171,11 @@ void RenderSelect::updateFromElement()
                     static_cast<KComboBox*>(m_widget)->insertItem(itemText, listIndex);
 #endif
             }
+            else if (listItems[listIndex]->id() == ID_HR) {
+                if (!m_useListBox) {
+                    static_cast<KComboBox*>(m_widget)->appendSeparator();
+                }
+            }
             else
                 KHTMLAssert(false);
             m_selectionChanged = true;
@@ -1275,12 +1280,10 @@ void RenderSelect::layout( )
         setIntrinsicHeight( s.height() );
     }
 
-    /// uuh, ignore the following line..
-    setNeedsLayout(true);
     RenderFormElement::layout();
 
     // and now disable the widget in case there is no <option> given
-    QMemArray<HTMLGenericFormElementImpl*> listItems = element()->listItems();
+    QMemArray<HTMLElementImpl*> listItems = element()->listItems();
 
     bool foundOption = false;
     for (uint i = 0; i < listItems.size() && !foundOption; i++)
@@ -1295,7 +1298,7 @@ void RenderSelect::slotSelected(int index)
 
     KHTMLAssert( !m_useListBox );
 
-    QMemArray<HTMLGenericFormElementImpl*> listItems = element()->listItems();
+    QMemArray<HTMLElementImpl*> listItems = element()->listItems();
     if(index >= 0 && index < int(listItems.size()))
     {
         bool found = ( listItems[index]->id() == ID_OPTION );
@@ -1343,14 +1346,17 @@ void RenderSelect::slotSelectionChanged()
 
     // don't use listItems() here as we have to avoid recalculations - changing the
     // option list will make use update options not in the way the user expects them
-    QMemArray<HTMLGenericFormElementImpl*> listItems = element()->m_listItems;
-    for ( unsigned i = 0; i < listItems.count(); i++ )
+    QMemArray<HTMLElementImpl*> listItems = element()->m_listItems;
+    int j = 0;
+    for ( unsigned i = 0; i < listItems.count(); i++ ) {
         // don't use setSelected() here because it will cause us to be called
         // again with updateSelection.
         if ( listItems[i]->id() == ID_OPTION )
             static_cast<HTMLOptionElementImpl*>( listItems[i] )
-                ->m_selected = static_cast<KListBox*>( m_widget )->isSelected( i );
-
+                ->m_selected = static_cast<KListBox*>( m_widget )->isSelected( j );
+        if ( listItems[i]->id() == ID_OPTION || listItems[i]->id() == ID_OPTGROUP)
+            ++j;
+    }
     element()->onChange();
 }
 
@@ -1383,14 +1389,18 @@ ComboBoxWidget *RenderSelect::createComboBox()
 
 void RenderSelect::updateSelection()
 {
-    QMemArray<HTMLGenericFormElementImpl*> listItems = element()->listItems();
+    QMemArray<HTMLElementImpl*> listItems = element()->listItems();
     int i;
     if (m_useListBox) {
         // if multi-select, we select only the new selected index
         KListBox *listBox = static_cast<KListBox*>(m_widget);
-        for (i = 0; i < int(listItems.size()); i++)
-            listBox->setSelected(i,listItems[i]->id() == ID_OPTION &&
+        int j = 0;
+        for (i = 0; i < int(listItems.size()); i++) {
+            listBox->setSelected(j,listItems[i]->id() == ID_OPTION &&
                                 static_cast<HTMLOptionElementImpl*>(listItems[i])->selected());
+            if (listItems[i]->id() == ID_OPTION || listItems[i]->id() == ID_OPTGROUP)
+                ++j;
+        }
     }
     else {
         bool found = false;
