@@ -1730,6 +1730,14 @@ int RenderObject::paddingRight() const
     return w;
 }
 
+int RenderObject::tabWidth() const
+{
+    if (style()->whiteSpace() != PRE)
+        return 0;
+        
+    return containingBlock()->tabWidth(true);
+}
+
 RenderCanvas* RenderObject::canvas() const
 {
     return static_cast<RenderCanvas*>(document()->renderer());
@@ -2042,9 +2050,6 @@ void RenderObject::recalcMinMaxWidths()
     
     RenderObject *child = firstChild();
     while( child ) {
-        // gcc sucks. if anybody knows a trick to get rid of the
-        // warning without adding an extra (unneeded) initialisation,
-        // go ahead
 	int cmin = 0;
 	int cmax = 0;
 	bool test = false;
