@@ -386,6 +386,8 @@ public:
     virtual short verticalPositionHint( bool firstLine ) const;
     // the offset of baseline from the top of the object.
     virtual short baselinePosition( bool firstLine, bool isRootLineBox=false ) const;
+    // width of tab character
+    int tabWidth() const;
 
     /*
      * Paint the object and its children, clipped by (x|y|w|h).
