@@ -42,6 +42,7 @@
 #include <qapplication.h>
 
 #include "dom/dom_misc.h"
+#include "misc/main_thread_malloc.h"
 #include "misc/khtmllayout.h"
 #include "misc/shared.h"
 #include "rendering/font.h"
@@ -52,6 +53,8 @@
     if (!(group->variable == value)) \
         group.access()->variable = value;
 
+class RenderArena;
+
 namespace DOM {
     class DOMStringImpl;
     class ShadowValueImpl;
@@ -134,6 +137,7 @@ private:
     DATA* data;
 };
 
+enum PseudoState { PseudoUnknown, PseudoNone, PseudoAnyLink, PseudoLink, PseudoVisited};
 
 //------------------------------------------------
 
@@ -190,7 +194,6 @@ enum EBorderStyle {
     BNONE, BHIDDEN, INSET, GROOVE, RIDGE, OUTSET, DOTTED, DASHED, SOLID, DOUBLE
 };
 
-
 class BorderValue
 {
 public:
@@ -233,6 +236,16 @@ public:
         _auto = false;
     }
     
+    bool operator==(const OutlineValue& o) const
+    {
+    	return width==o.width && style==o.style && color==o.color && _offset == o._offset && _auto == o._auto;
+    }
+    
+    bool operator!=(const OutlineValue& o) const
+    {
+        return !(*this == o);
+    }
+    
     int _offset;
     bool _auto;
 };
@@ -261,7 +274,7 @@ struct CollapsedBorderValue
     EBorderPrecedence precedence;    
 };
 
-class BorderData : public Shared<BorderData>
+class BorderData
 {
 public:
     BorderValue left;
@@ -281,6 +294,8 @@ public:
 
 };
 
+enum EMarginCollapse { MCOLLAPSE, MSEPARATE, MDISCARD };
+
 class StyleSurroundData : public Shared<StyleSurroundData>
 {
 public:
@@ -334,11 +349,34 @@ public:
     bool z_auto : 1;
 };
 
+//------------------------------------------------
+// Dashboard region attributes. Not inherited.
+
+#if APPLE_CHANGES
+struct StyleDashboardRegion
+{
+    QString label;
+    LengthBox offset;
+    int type;
+    
+    enum {
+        None,
+        Circle,
+        Rectangle
+    };
+
+    bool operator==(const StyleDashboardRegion& o) const
+    {
+        return type == o.type && offset == o.offset && label == o.label;
+    }
+};
+#endif
+
 //------------------------------------------------
 // Random visual rendering model attributes. Not inherited.
 
 enum EOverflow {
-    OVISIBLE, OHIDDEN, OSCROLL, OAUTO, OMARQUEE
+    OVISIBLE, OHIDDEN, OSCROLL, OAUTO, OMARQUEE, OOVERLAY
 };
 
 enum EVerticalAlign {
@@ -373,7 +411,10 @@ public:
 		 colspan == o.colspan &&
 		 counter_increment == o.counter_increment &&
 		 counter_reset == o.counter_reset &&
-		 palette == o.palette && textDecoration == o.textDecoration);
+#if !APPLE_CHANGES
+		 palette == o.palette &&
+#endif
+                 textDecoration == o.textDecoration);
     }
     bool operator!=( const StyleVisualData &o ) const {
         return !(*this == o);
@@ -388,7 +429,9 @@ public:
     short counter_increment; //ok, so these are not visual mode spesific
     short counter_reset;     //can't go to inherited, since these are not inherited
 
+#if !APPLE_CHANGES
     QPalette palette;      //widget styling with IE attributes
+#endif
 };
 
 //------------------------------------------------
@@ -396,7 +439,81 @@ enum EBackgroundRepeat {
     REPEAT, REPEAT_X, REPEAT_Y, NO_REPEAT
 };
 
+struct BackgroundLayer {
+public:
+    BackgroundLayer();
+    ~BackgroundLayer();
 
+    MAIN_THREAD_ALLOCATED;
+
+    CachedImage* backgroundImage() const { return m_image; }
+    Length backgroundXPosition() const { return m_xPosition; }
+    Length backgroundYPosition() const { return m_yPosition; }
+    bool backgroundAttachment() const { return m_bgAttachment; }
+    EBackgroundRepeat backgroundRepeat() const { return m_bgRepeat; }
+    BackgroundLayer* next() const { return m_next; }
+    BackgroundLayer* next() { return m_next; }
+
+    bool isBackgroundImageSet() const { return m_imageSet; }
+    bool isBackgroundXPositionSet() const { return m_xPosSet; }
+    bool isBackgroundYPositionSet() const { return m_yPosSet; }
+    bool isBackgroundAttachmentSet() const { return m_attachmentSet; }
+    bool isBackgroundRepeatSet() const { return m_repeatSet; }
+
+    void setBackgroundImage(CachedImage* i) { m_image = i; m_imageSet = true; }
+    void setBackgroundXPosition(const Length& l) { m_xPosition = l; m_xPosSet = true; }
+    void setBackgroundYPosition(const Length& l) { m_yPosition = l; m_yPosSet = true; }
+    void setBackgroundAttachment(bool b) { m_bgAttachment = b; m_attachmentSet = true; }
+    void setBackgroundRepeat(EBackgroundRepeat r) { m_bgRepeat = r; m_repeatSet = true; }
+    
+    void clearBackgroundImage() { m_imageSet = false; }
+    void clearBackgroundXPosition() { m_xPosSet = false; }
+    void clearBackgroundYPosition() { m_yPosSet = false; }
+    void clearBackgroundAttachment() { m_attachmentSet = false; }
+    void clearBackgroundRepeat() { m_repeatSet = false; }
+
+    void setNext(BackgroundLayer* n) { if (m_next != n) { delete m_next; m_next = n; } }
+
+    BackgroundLayer& operator=(const BackgroundLayer& o);    
+    BackgroundLayer(const BackgroundLayer& o);
+
+    bool operator==(const BackgroundLayer& o) const;
+    bool operator!=(const BackgroundLayer& o) const {
+        return !(*this == o);
+    }
+
+    bool containsImage(CachedImage* c) const { if (c == m_image) return true; if (m_next) return m_next->containsImage(c); return false; }
+    
+    bool hasImage() const {
+        if (m_image)
+            return true;
+        return m_next ? m_next->hasImage() : false;
+    }
+    bool hasFixedImage() const {
+        if (m_image && !m_bgAttachment)
+            return true;
+        return m_next ? m_next->hasFixedImage() : false;
+    }
+
+    void fillUnsetProperties();
+    void cullEmptyLayers();
+
+    CachedImage* m_image;
+
+    Length m_xPosition;
+    Length m_yPosition;
+
+    bool m_bgAttachment : 1;
+    EBackgroundRepeat m_bgRepeat : 2;
+
+    bool m_imageSet : 1;
+    bool m_attachmentSet : 1;
+    bool m_repeatSet : 1;
+    bool m_xPosSet : 1;
+    bool m_yPosSet : 1;
+
+    BackgroundLayer* m_next;
+};
 
 class StyleBackgroundData : public Shared<StyleBackgroundData>
 {
@@ -410,12 +527,9 @@ public:
 	return !(*this == o);
     }
 
-    QColor color;
-    CachedImage *image;
-
-    Length x_position;
-    Length y_position;
-    OutlineValue outline;
+    BackgroundLayer m_background;
+    QColor m_color;
+    OutlineValue m_outline;
 };
 
 //------------------------------------------------
@@ -456,7 +570,6 @@ class StyleFlexibleBoxData : public Shared<StyleFlexibleBoxData>
 {
 public:
     StyleFlexibleBoxData();
-    ~StyleFlexibleBoxData() {}
     StyleFlexibleBoxData(const StyleFlexibleBoxData& o);
 
     bool operator==(const StyleFlexibleBoxData& o) const;
@@ -467,8 +580,7 @@ public:
     float flex;
     unsigned int flex_group;
     unsigned int ordinal_group;
-    int flexed_height; // Not an actual CSS property. Vertical flexing has to use this as a cache.
-    
+
     EBoxAlignment align : 3;
     EBoxAlignment pack: 3;
     EBoxOrient orient: 1;
@@ -483,6 +595,8 @@ struct ShadowData {
     
     ~ShadowData() { delete next; }
 
+    MAIN_THREAD_ALLOCATED;
+
     bool operator==(const ShadowData& o) const;
     bool operator!=(const ShadowData &o) const {
         return !(*this == o);
@@ -495,6 +609,65 @@ struct ShadowData {
     ShadowData* next;
 };
 
+#ifndef KHTML_NO_XBL
+struct BindingURI {
+    BindingURI(DOM::DOMStringImpl*);
+    ~BindingURI();
+
+    BindingURI* copy();
+
+    bool operator==(const BindingURI& o) const;
+    bool operator!=(const BindingURI& o) const {
+        return !(*this == o);
+    }
+    
+    BindingURI* next() { return m_next; }
+    void setNext(BindingURI* n) { m_next = n; }
+    
+    DOM::DOMStringImpl* uri() { return m_uri; }
+    
+    BindingURI* m_next;
+    DOM::DOMStringImpl* m_uri;
+};
+#endif
+
+//------------------------------------------------
+// CSS3 User Modify Properties
+
+enum EUserModify {
+    READ_ONLY, READ_WRITE
+};
+
+// CSS3 User Drag Values
+
+enum EUserDrag {
+    DRAG_AUTO, DRAG_NONE, DRAG_ELEMENT
+};
+
+// CSS3 User Select Values
+
+enum EUserSelect {
+    SELECT_AUTO, SELECT_NONE, SELECT_TEXT
+};
+
+// Word Break Values. Matches WinIE, rather than CSS3
+
+enum EWordWrap {
+    WBNORMAL, BREAK_WORD
+};
+
+enum ENBSPMode {
+    NBNORMAL, SPACE
+};
+
+enum EKHTMLLineBreak {
+    LBNORMAL, AFTER_WHITE_SPACE
+};
+
+enum EMatchNearestMailBlockquoteColor {
+    BCNORMAL, MATCH
+};
+
 // This struct is for rarely used non-inherited CSS3 properties.  By grouping them together,
 // we save space, and only allocate this object when someone actually uses
 // a non-inherited CSS3 property.
@@ -502,22 +675,40 @@ class StyleCSS3NonInheritedData : public Shared<StyleCSS3NonInheritedData>
 {
 public:
     StyleCSS3NonInheritedData();
-    ~StyleCSS3NonInheritedData() {}
+    ~StyleCSS3NonInheritedData();
     StyleCSS3NonInheritedData(const StyleCSS3NonInheritedData& o);
 
+#ifndef KHTML_NO_XBL
+    bool bindingsEquivalent(const StyleCSS3NonInheritedData& o) const;
+#endif
+
     bool operator==(const StyleCSS3NonInheritedData& o) const;
     bool operator!=(const StyleCSS3NonInheritedData &o) const {
         return !(*this == o);
     }
     
+#if APPLE_CHANGES
+    int lineClamp;         // An Apple extension.  Not really CSS3 but not worth making a new struct over.
+    QValueList<StyleDashboardRegion> m_dashboardRegions;
+#endif
     float opacity;         // Whether or not we're transparent.
     DataRef<StyleFlexibleBoxData> flexibleBox; // Flexible box properties 
     DataRef<StyleMarqueeData> marquee; // Marquee properties
+    EUserDrag userDrag : 2; // Whether or not a drag can be initiated by this element.
+    EUserSelect userSelect : 2;  // Whether or not the element is selectable.
+    bool textOverflow : 1; // Whether or not lines that spill out should be truncated with "..."
+    EMarginCollapse marginTopCollapse : 2;
+    EMarginCollapse marginBottomCollapse : 2;
+    EMatchNearestMailBlockquoteColor matchNearestMailBlockquoteColor : 1;    
+
+#ifndef KHTML_NO_XBL
+    BindingURI* bindingURI; // The XBL binding URI list.
+#endif
 };
 
 // This struct is for rarely used inherited CSS3 properties.  By grouping them together,
 // we save space, and only allocate this object when someone actually uses
-// a non-inherited CSS3 property.
+// an inherited CSS3 property.
 class StyleCSS3InheritedData : public Shared<StyleCSS3InheritedData>
 {
 public:
@@ -532,7 +723,14 @@ public:
     bool shadowDataEquivalent(const StyleCSS3InheritedData& o) const;
 
     ShadowData* textShadow;  // Our text shadow information for shadowed text drawing.
-
+    EUserModify userModify : 2; // Flag used for editing state
+    EWordWrap wordWrap : 1;    // Flag used for word wrap
+    ENBSPMode nbspMode : 1;    
+    EKHTMLLineBreak khtmlLineBreak : 1;    
+#if APPLE_CHANGES
+    bool textSizeAdjust : 1;    // An Apple extension.  Not really CSS3 but not worth making a new struct over.
+#endif
+    
 private:
     StyleCSS3InheritedData &operator=(const StyleCSS3InheritedData &);
 };
@@ -650,7 +848,7 @@ struct ContentData {
 
     ContentData* _nextContent;
 };
-    
+
 //------------------------------------------------
 
 enum EDisplay {
@@ -661,7 +859,7 @@ enum EDisplay {
     TABLE_CAPTION, BOX, INLINE_BOX, NONE
 };
 
-class RenderStyle : public Shared<RenderStyle>
+class RenderStyle
 {
     friend class CSSStyleSelector;
 public:
@@ -670,6 +868,25 @@ public:
     // static pseudo styles. Dynamic ones are produced on the fly.
     enum PseudoId { NOPSEUDO, FIRST_LINE, FIRST_LETTER, BEFORE, AFTER, SELECTION, FIRST_LINE_INHERITED };
 
+    void ref() { m_ref++;  }
+    void deref(RenderArena* arena) { 
+	if (m_ref) m_ref--; 
+	if (!m_ref)
+	    arenaDelete(arena);
+    }
+    bool hasOneRef() { return m_ref==1; }
+    int refCount() const { return m_ref; }
+    
+    // Overloaded new operator.  Derived classes must override operator new
+    // in order to allocate out of the RenderArena.
+    void* operator new(size_t sz, RenderArena* renderArena) throw();    
+    
+    // Overridden to prevent the normal delete from being called.
+    void operator delete(void* ptr, size_t sz);
+    
+private:
+    void arenaDelete(RenderArena *arena);
+
 protected:
 
 // !START SYNC!: Keep this in sync with the copy constructor in render_style.cpp
@@ -693,7 +910,7 @@ protected:
                    (_box_direction == other._box_direction) &&
                    (_visuallyOrdered == other._visuallyOrdered) &&
                    (_htmlHacks == other._htmlHacks) &&
-                   (_should_correct_text_color == other._should_correct_text_color);
+                   (_force_backgrounds_to_white == other._force_backgrounds_to_white);
 	}
         
 	bool operator!=( const InheritedFlags &other ) const {
@@ -716,7 +933,7 @@ protected:
               // non CSS2 inherited
               bool _visuallyOrdered : 1;
               bool _htmlHacks :1;
-              bool _should_correct_text_color : 1;
+              bool _force_backgrounds_to_white : 1;
     } inherited_flags;
 
 // don't inherit
@@ -736,6 +953,7 @@ protected:
             (_styleType == other._styleType) &&
             (_affectedByHover == other._affectedByHover) &&
             (_affectedByActive == other._affectedByActive) &&
+            (_affectedByDrag == other._affectedByDrag) &&
             (_pseudoBits == other._pseudoBits) &&
             (_unicodeBidi == other._unicodeBidi);
 	}
@@ -747,7 +965,6 @@ protected:
         EDisplay _effectiveDisplay : 5;
         EDisplay _originalDisplay : 5;
         EBackgroundRepeat _bg_repeat : 2;
-        bool _bg_attachment : 1;
         EOverflow _overflow : 4 ;
         EVerticalAlign _vertical_align : 4;
         EClear _clear : 2;
@@ -761,6 +978,7 @@ protected:
         PseudoId _styleType : 3;
         bool _affectedByHover : 1;
         bool _affectedByActive : 1;
+        bool _affectedByDrag : 1;
         int _pseudoBits : 6;
         EUnicodeBidi _unicodeBidi : 2;
     } noninherited_flags;
@@ -771,7 +989,7 @@ protected:
     DataRef<StyleBackgroundData> background;
     DataRef<StyleSurroundData> surround;
     DataRef<StyleCSS3NonInheritedData> css3NonInheritedData;
-    
+
 // inherited attributes
     DataRef<StyleCSS3InheritedData> css3InheritedData;
     DataRef<StyleInheritedData> inherited;
@@ -782,6 +1000,12 @@ protected:
     // added this here, so we can get rid of the vptr in this class.
     // makes up for the same size.
     ContentData *content;
+
+    PseudoState m_pseudoState : 3;
+    bool m_affectedByAttributeSelectors : 1;
+    
+    int m_ref;
+    
 // !END SYNC!
 
 // static default style
@@ -805,11 +1029,9 @@ protected:
 	inherited_flags._visuallyOrdered = false;
 	inherited_flags._htmlHacks=false;
         inherited_flags._box_direction = initialBoxDirection();
-        inherited_flags._should_correct_text_color = false;
+        inherited_flags._force_backgrounds_to_white = false;
         
 	noninherited_flags._effectiveDisplay = noninherited_flags._originalDisplay = initialDisplay();
-	noninherited_flags._bg_repeat = initialBackgroundRepeat();
-	noninherited_flags._bg_attachment = initialBackgroundAttachment();
 	noninherited_flags._overflow = initialOverflow();
 	noninherited_flags._vertical_align = initialVerticalAlign();
 	noninherited_flags._clear = initialClear();
@@ -821,6 +1043,7 @@ protected:
 	noninherited_flags._styleType = NOPSEUDO;
         noninherited_flags._affectedByHover = false;
         noninherited_flags._affectedByActive = false;
+        noninherited_flags._affectedByDrag = false;
         noninherited_flags._pseudoBits = 0;
 	noninherited_flags._unicodeBidi = initialUnicodeBidi();
     }
@@ -840,13 +1063,14 @@ public:
 
     RenderStyle* getPseudoStyle(PseudoId pi);
     void addPseudoStyle(RenderStyle* pseudo);
-    void removePseudoStyle(PseudoId pi);
 
     bool affectedByHoverRules() const { return  noninherited_flags._affectedByHover; }
     bool affectedByActiveRules() const { return  noninherited_flags._affectedByActive; }
+    bool affectedByDragRules() const { return  noninherited_flags._affectedByDrag; }
 
     void setAffectedByHoverRules(bool b) {  noninherited_flags._affectedByHover = b; }
     void setAffectedByActiveRules(bool b) {  noninherited_flags._affectedByActive = b; }
+    void setAffectedByDragRules(bool b) {  noninherited_flags._affectedByDrag = b; }
  
     bool operator==(const RenderStyle& other) const;
     bool        isFloating() const { return !(noninherited_flags._floating == FNONE); }
@@ -854,6 +1078,11 @@ public:
     bool        hasBorder() const { return surround->border.hasBorder(); }
     bool        hasOffset() const { return surround->offset.nonZero(); }
 
+    bool hasBackground() const { if (backgroundColor().isValid() && qAlpha(backgroundColor().rgb()) > 0)
+                                    return true;
+                                 return background->m_background.hasImage(); }
+    bool hasFixedBackgroundImage() const { return background->m_background.hasFixedImage(); }
+
     bool visuallyOrdered() const { return inherited_flags._visuallyOrdered; }
     void setVisuallyOrdered(bool b) {  inherited_flags._visuallyOrdered = b; }
 
@@ -888,36 +1117,33 @@ public:
     const BorderValue& borderBottom() const { return surround->border.bottom; }
 
     unsigned short  borderLeftWidth() const
-    { if( surround->border.left.style == BNONE) return 0; return surround->border.left.width; }
+    { if( surround->border.left.style == BNONE || surround->border.left.style == BHIDDEN) return 0; return surround->border.left.width; }
     EBorderStyle    borderLeftStyle() const { return surround->border.left.style; }
     const QColor &  borderLeftColor() const { return surround->border.left.color; }
     bool borderLeftIsTransparent() const { return surround->border.left.isTransparent(); }
     unsigned short  borderRightWidth() const
-    { if (surround->border.right.style == BNONE) return 0; return surround->border.right.width; }
+    { if (surround->border.right.style == BNONE || surround->border.right.style == BHIDDEN) return 0; return surround->border.right.width; }
     EBorderStyle    borderRightStyle() const {  return surround->border.right.style; }
     const QColor &  	    borderRightColor() const {  return surround->border.right.color; }
     bool borderRightIsTransparent() const { return surround->border.right.isTransparent(); }
     unsigned short  borderTopWidth() const
-    { if(surround->border.top.style == BNONE) return 0; return surround->border.top.width; }
+    { if(surround->border.top.style == BNONE || surround->border.top.style == BHIDDEN) return 0; return surround->border.top.width; }
     EBorderStyle    borderTopStyle() const {return surround->border.top.style; }
     const QColor &  borderTopColor() const {  return surround->border.top.color; }
     bool borderTopIsTransparent() const { return surround->border.top.isTransparent(); }
     unsigned short  borderBottomWidth() const
-    { if(surround->border.bottom.style == BNONE) return 0; return surround->border.bottom.width; }
+    { if(surround->border.bottom.style == BNONE || surround->border.bottom.style == BHIDDEN) return 0; return surround->border.bottom.width; }
     EBorderStyle    borderBottomStyle() const {  return surround->border.bottom.style; }
     const QColor &  	    borderBottomColor() const {  return surround->border.bottom.color; }
     bool borderBottomIsTransparent() const { return surround->border.bottom.isTransparent(); }
     
     unsigned short outlineSize() const { return outlineWidth() + outlineOffset(); }
-    unsigned short outlineWidth() const { if (background->outline.style == BNONE) return 0; return background->outline.width; }
-    EBorderStyle    outlineStyle() const {  return background->outline.style; }
-    bool outlineStyleIsAuto() const { return background->outline._auto; }
-    const QColor &  	    outlineColor() const {  return background->outline.color; }
+    unsigned short outlineWidth() const { if (background->m_outline.style == BNONE || background->m_outline.style == BHIDDEN) return 0; return background->m_outline.width; }
+    EBorderStyle    outlineStyle() const {  return background->m_outline.style; }
+    bool outlineStyleIsAuto() const { return background->m_outline._auto; }
+    const QColor &  	    outlineColor() const {  return background->m_outline.color; }
 
     EOverflow overflow() const { return  noninherited_flags._overflow; }
-    bool hidesOverflow() const { return overflow() != OVISIBLE; }
-    bool scrollsOverflow() const { return overflow() == OSCROLL || overflow() == OAUTO; }
-
     EVisibility visibility() const { return inherited_flags._visibility; }
     EVerticalAlign verticalAlign() const { return  noninherited_flags._vertical_align; }
     Length verticalAlignLength() const { return box->vertical_align; }
@@ -926,6 +1152,7 @@ public:
     Length clipRight() const { return visual->clip.right; }
     Length clipTop() const { return visual->clip.top; }
     Length clipBottom() const { return visual->clip.bottom; }
+    LengthBox clip() const { return visual->clip; }
     bool hasClip() const { return visual->hasClip; }
     
     EUnicodeBidi unicodeBidi() const { return noninherited_flags._unicodeBidi; }
@@ -954,13 +1181,14 @@ public:
 
     EWhiteSpace whiteSpace() const { return inherited_flags._white_space; }
 
-
-    const QColor & backgroundColor() const { return background->color; }
-    CachedImage *backgroundImage() const { return background->image; }
-    EBackgroundRepeat backgroundRepeat() const { return  noninherited_flags._bg_repeat; }
-    bool backgroundAttachment() const { return  noninherited_flags._bg_attachment; }
-    Length backgroundXPosition() const { return background->x_position; }
-    Length backgroundYPosition() const { return background->y_position; }
+    const QColor & backgroundColor() const { return background->m_color; }
+    CachedImage *backgroundImage() const { return background->m_background.m_image; }
+    EBackgroundRepeat backgroundRepeat() const { return background->m_background.m_bgRepeat; }
+    bool backgroundAttachment() const { return background->m_background.m_bgAttachment; }
+    Length backgroundXPosition() const { return background->m_background.m_xPosition; }
+    Length backgroundYPosition() const { return background->m_background.m_yPosition; }
+    BackgroundLayer* accessBackgroundLayers() { return &(background.access()->m_background); }
+    const BackgroundLayer* backgroundLayers() const { return &(background->m_background); }
 
     // returns true for collapsing borders, false for separate borders
     bool borderCollapse() const { return inherited_flags._border_collapse; }
@@ -997,27 +1225,45 @@ public:
     EPageBreak pageBreakAfter() const { return noninherited_flags._page_break_after; }
     
     // CSS3 Getter Methods
+#ifndef KHTML_NO_XBL
+    BindingURI* bindingURIs() const { return css3NonInheritedData->bindingURI; }
+#endif
     int outlineOffset() const { 
-        if (background->outline.style == BNONE) return 0; return background->outline._offset;
+        if (background->m_outline.style == BNONE || background->m_outline.style == BHIDDEN) return 0; return background->m_outline._offset;
     }
     ShadowData* textShadow() const { return css3InheritedData->textShadow; }
-    float opacity() { return css3NonInheritedData->opacity; }
-    EBoxAlignment boxAlign() { return css3NonInheritedData->flexibleBox->align; }
-    EBoxDirection boxDirection() { return inherited_flags._box_direction; }
+    float opacity() const { return css3NonInheritedData->opacity; }
+    EBoxAlignment boxAlign() const { return css3NonInheritedData->flexibleBox->align; }
+    EBoxDirection boxDirection() const { return inherited_flags._box_direction; }
     float boxFlex() { return css3NonInheritedData->flexibleBox->flex; }
-    unsigned int boxFlexGroup() { return css3NonInheritedData->flexibleBox->flex_group; }
+    unsigned int boxFlexGroup() const { return css3NonInheritedData->flexibleBox->flex_group; }
     EBoxLines boxLines() { return css3NonInheritedData->flexibleBox->lines; }
-    unsigned int boxOrdinalGroup() { return css3NonInheritedData->flexibleBox->ordinal_group; }
-    EBoxOrient boxOrient() { return css3NonInheritedData->flexibleBox->orient; }
-    EBoxAlignment boxPack() { return css3NonInheritedData->flexibleBox->pack; }
-    int boxFlexedHeight() { return css3NonInheritedData->flexibleBox->flexed_height; }
-    Length marqueeIncrement() { return css3NonInheritedData->marquee->increment; }
-    int marqueeSpeed() { return css3NonInheritedData->marquee->speed; }
-    int marqueeLoopCount() { return css3NonInheritedData->marquee->loops; }
-    EMarqueeBehavior marqueeBehavior() { return css3NonInheritedData->marquee->behavior; }
-    EMarqueeDirection marqueeDirection() { return css3NonInheritedData->marquee->direction; }
+    unsigned int boxOrdinalGroup() const { return css3NonInheritedData->flexibleBox->ordinal_group; }
+    EBoxOrient boxOrient() const { return css3NonInheritedData->flexibleBox->orient; }
+    EBoxAlignment boxPack() const { return css3NonInheritedData->flexibleBox->pack; }
+    Length marqueeIncrement() const { return css3NonInheritedData->marquee->increment; }
+    int marqueeSpeed() const { return css3NonInheritedData->marquee->speed; }
+    int marqueeLoopCount() const { return css3NonInheritedData->marquee->loops; }
+    EMarqueeBehavior marqueeBehavior() const { return css3NonInheritedData->marquee->behavior; }
+    EMarqueeDirection marqueeDirection() const { return css3NonInheritedData->marquee->direction; }
+    EUserModify userModify() const { return css3InheritedData->userModify; }
+    EUserDrag userDrag() const { return css3NonInheritedData->userDrag; }
+    EUserSelect userSelect() const { return css3NonInheritedData->userSelect; }
+    bool textOverflow() const { return css3NonInheritedData->textOverflow; }
+    EMarginCollapse marginTopCollapse() const { return css3NonInheritedData->marginTopCollapse; }
+    EMarginCollapse marginBottomCollapse() const { return css3NonInheritedData->marginBottomCollapse; }
+    EWordWrap wordWrap() const { return css3InheritedData->wordWrap; }
+    ENBSPMode nbspMode() const { return css3InheritedData->nbspMode; }
+    EKHTMLLineBreak khtmlLineBreak() const { return css3InheritedData->khtmlLineBreak; }
+    EMatchNearestMailBlockquoteColor matchNearestMailBlockquoteColor() const { return css3NonInheritedData->matchNearestMailBlockquoteColor; }
     // End CSS3 Getters
 
+#if APPLE_CHANGES
+    // Apple-specific property getter methods
+    int lineClamp() const { return css3NonInheritedData->lineClamp; }
+    bool textSizeAdjust() const { return css3InheritedData->textSizeAdjust; }
+#endif
+
 // attribute setter methods
 
     void setDisplay(EDisplay v) {  noninherited_flags._effectiveDisplay = v; }
@@ -1038,12 +1284,32 @@ public:
     void setMinHeight(Length v) { SET_VAR(box,min_height,v) }
     void setMaxHeight(Length v) { SET_VAR(box,max_height,v) }
 
+#if APPLE_CHANGES
+    QValueList<StyleDashboardRegion> dashboardRegions() const { return css3NonInheritedData->m_dashboardRegions; }
+    void setDashboardRegions(QValueList<StyleDashboardRegion> regions) { SET_VAR(css3NonInheritedData,m_dashboardRegions,regions); }
+    void setDashboardRegion (int type, QString label, Length t, Length r, Length b, Length l, bool append) {
+        StyleDashboardRegion region;
+        region.label = label;
+        region.offset.top  = t;
+        region.offset.right = r;
+        region.offset.bottom = b;
+        region.offset.left = l;
+        region.type = type;
+        if (!append) {
+            css3NonInheritedData.access()->m_dashboardRegions.clear ();
+        }
+        css3NonInheritedData.access()->m_dashboardRegions.append (region);
+    }
+#endif
+
     void resetBorderTop() { SET_VAR(surround, border.top, BorderValue()) }
     void resetBorderRight() { SET_VAR(surround, border.right, BorderValue()) }
     void resetBorderBottom() { SET_VAR(surround, border.bottom, BorderValue()) }
     void resetBorderLeft() { SET_VAR(surround, border.left, BorderValue()) }
-    void resetOutline() { SET_VAR(background, outline, OutlineValue()) }
+    void resetOutline() { SET_VAR(background, m_outline, OutlineValue()) }
     
+    void setBackgroundColor(const QColor& v)    { SET_VAR(background, m_color, v) }
+
     void setBorderLeftWidth(unsigned short v)   {  SET_VAR(surround,border.left.width,v) }
     void setBorderLeftStyle(EBorderStyle v)     {  SET_VAR(surround,border.left.style,v) }
     void setBorderLeftColor(const QColor & v)   {  SET_VAR(surround,border.left.color,v) }
@@ -1056,13 +1322,13 @@ public:
     void setBorderBottomWidth(unsigned short v) {  SET_VAR(surround,border.bottom.width,v) }
     void setBorderBottomStyle(EBorderStyle v)   {  SET_VAR(surround,border.bottom.style,v) }
     void setBorderBottomColor(const QColor & v) {  SET_VAR(surround,border.bottom.color,v) }
-    void setOutlineWidth(unsigned short v) {  SET_VAR(background,outline.width,v) }
+    void setOutlineWidth(unsigned short v) {  SET_VAR(background,m_outline.width,v) }
     void setOutlineStyle(EBorderStyle v, bool isAuto = false)   
     {  
-        SET_VAR(background,outline.style,v)
-        SET_VAR(background,outline._auto, isAuto)
+        SET_VAR(background,m_outline.style,v)
+        SET_VAR(background,m_outline._auto, isAuto)
     }
-    void setOutlineColor(const QColor & v) {  SET_VAR(background,outline.color,v) }
+    void setOutlineColor(const QColor & v) {  SET_VAR(background,m_outline.color,v) }
 
     void setOverflow(EOverflow v) {  noninherited_flags._overflow = v; }
     void setVisibility(EVisibility v) { inherited_flags._visibility = v; }
@@ -1106,12 +1372,9 @@ public:
     void setWordSpacing(int v) { SET_VAR(inherited,font.wordSpacing,v) }
     void setLetterSpacing(int v) { SET_VAR(inherited,font.letterSpacing,v) }
 
-    void setBackgroundColor(const QColor & v) {  SET_VAR(background,color,v) }
-    void setBackgroundImage(CachedImage *v) {  SET_VAR(background,image,v) }
-    void setBackgroundRepeat(EBackgroundRepeat v) {  noninherited_flags._bg_repeat = v; }
-    void setBackgroundAttachment(bool scroll) {  noninherited_flags._bg_attachment = scroll; }
-    void setBackgroundXPosition(Length v) {  SET_VAR(background,x_position,v) }
-    void setBackgroundYPosition(Length v) {  SET_VAR(background,y_position,v) }
+    void clearBackgroundLayers() { background.access()->m_background = BackgroundLayer(); }
+    void inheritBackgroundLayers(const BackgroundLayer& parent) { background.access()->m_background = parent; }
+    void adjustBackgroundLayers();
 
     void setBorderCollapse(bool collapse) { inherited_flags._border_collapse = collapse; }
     void setHorizontalBorderSpacing(short v) { SET_VAR(inherited,horizontal_border_spacing,v) }
@@ -1142,16 +1405,16 @@ public:
     void setCursor( ECursor c ) { inherited_flags._cursor_style = c; }
     void setCursorImage( CachedImage *v ) { SET_VAR(inherited,cursor_image,v) }
 
-    bool shouldCorrectTextColor() const { return inherited_flags._should_correct_text_color; }
-    void setShouldCorrectTextColor(bool b=true) { inherited_flags._should_correct_text_color = b; }
+    bool forceBackgroundsToWhite() const { return inherited_flags._force_backgrounds_to_white; }
+    void setForceBackgroundsToWhite(bool b=true) { inherited_flags._force_backgrounds_to_white = b; }
 
     bool htmlHacks() const { return inherited_flags._htmlHacks; }
     void setHtmlHacks(bool b=true) { inherited_flags._htmlHacks = b; }
 
     bool hasAutoZIndex() { return box->z_auto; }
-    void setHasAutoZIndex() { SET_VAR(box, z_auto, true) }
+    void setHasAutoZIndex() { SET_VAR(box, z_auto, true); SET_VAR(box, z_index, 0) }
     int zIndex() const { return box->z_index; }
-    void setZIndex(int v) { SET_VAR(box, z_auto, false); SET_VAR(box,z_index,v) }
+    void setZIndex(int v) { SET_VAR(box, z_auto, false); SET_VAR(box, z_index, v) }
 
     void setWidows(short w) { SET_VAR(inherited, widows, w); }
     void setOrphans(short o) { SET_VAR(inherited, orphans, o); }
@@ -1160,7 +1423,17 @@ public:
     void setPageBreakAfter(EPageBreak b) { noninherited_flags._page_break_after = b; }
     
     // CSS3 Setters
-    void setOutlineOffset(unsigned short v) {  SET_VAR(background,outline._offset,v) }
+#ifndef KHTML_NO_XBL
+    void deleteBindingURIs() { 
+        delete css3NonInheritedData->bindingURI; 
+        SET_VAR(css3NonInheritedData, bindingURI, 0);
+    }
+    void inheritBindingURIs(BindingURI* other) {
+        SET_VAR(css3NonInheritedData, bindingURI, other->copy());
+    }
+    void addBindingURI(DOM::DOMStringImpl* uri);
+#endif
+    void setOutlineOffset(unsigned short v) { SET_VAR(background, m_outline._offset, v) }
     void setTextShadow(ShadowData* val, bool add=false);
     void setOpacity(float f) { SET_VAR(css3NonInheritedData, opacity, f); }
     void setBoxAlign(EBoxAlignment a) { SET_VAR(css3NonInheritedData.access()->flexibleBox, align, a); }
@@ -1171,20 +1444,37 @@ public:
     void setBoxOrdinalGroup(unsigned int og) { SET_VAR(css3NonInheritedData.access()->flexibleBox, ordinal_group, og); }
     void setBoxOrient(EBoxOrient o) { SET_VAR(css3NonInheritedData.access()->flexibleBox, orient, o); }
     void setBoxPack(EBoxAlignment p) { SET_VAR(css3NonInheritedData.access()->flexibleBox, pack, p); }
-    void setBoxFlexedHeight(int h) { SET_VAR(css3NonInheritedData.access()->flexibleBox, flexed_height, h); }
     void setMarqueeIncrement(const Length& f) { SET_VAR(css3NonInheritedData.access()->marquee, increment, f); }
     void setMarqueeSpeed(int f) { SET_VAR(css3NonInheritedData.access()->marquee, speed, f); }
     void setMarqueeDirection(EMarqueeDirection d) { SET_VAR(css3NonInheritedData.access()->marquee, direction, d); }
     void setMarqueeBehavior(EMarqueeBehavior b) { SET_VAR(css3NonInheritedData.access()->marquee, behavior, b); }
     void setMarqueeLoopCount(int i) { SET_VAR(css3NonInheritedData.access()->marquee, loops, i); }
+    void setUserModify(EUserModify u) { SET_VAR(css3InheritedData, userModify, u); }
+    void setUserDrag(EUserDrag d) { SET_VAR(css3NonInheritedData, userDrag, d); }
+    void setUserSelect(EUserSelect s) { SET_VAR(css3NonInheritedData, userSelect, s); }
+    void setTextOverflow(bool b) { SET_VAR(css3NonInheritedData, textOverflow, b); }
+    void setMarginTopCollapse(EMarginCollapse c) { SET_VAR(css3NonInheritedData, marginTopCollapse, c); }
+    void setMarginBottomCollapse(EMarginCollapse c) { SET_VAR(css3NonInheritedData, marginBottomCollapse, c); }
+    void setWordWrap(EWordWrap b) { SET_VAR(css3InheritedData, wordWrap, b); }
+    void setNBSPMode(ENBSPMode b) { SET_VAR(css3InheritedData, nbspMode, b); }
+    void setKHTMLLineBreak(EKHTMLLineBreak b) { SET_VAR(css3InheritedData, khtmlLineBreak, b); }
+    void setMatchNearestMailBlockquoteColor(EMatchNearestMailBlockquoteColor c)  { SET_VAR(css3NonInheritedData, matchNearestMailBlockquoteColor, c); }
     // End CSS3 Setters
-    
+   
+#if APPLE_CHANGES
+    // Apple-specific property setters
+    void setLineClamp(int c) { SET_VAR(css3NonInheritedData, lineClamp, c); }
+    void setTextSizeAdjust(bool b) { SET_VAR(css3InheritedData, textSizeAdjust, b); }
+#endif
+
+#if !APPLE_CHANGES
     QPalette palette() const { return visual->palette; }
     void setPaletteColor(QPalette::ColorGroup g, QColorGroup::ColorRole r, const QColor& c);
     void resetPalette() // Called when the desktop color scheme changes.
     {
         const_cast<StyleVisualData *>(visual.get())->palette = QApplication::palette();
     }
+#endif
 
     ContentData* contentData() { return content; }
     bool contentDataEquivalent(RenderStyle* otherStyle);
@@ -1193,18 +1483,33 @@ public:
 
     bool inheritedNotEqual( RenderStyle *other ) const;
 
-    enum Diff { Equal, NonVisible = Equal, Visible, Position, Layout, CbLayout };
+    // The difference between two styles.  The following values are used:
+    // (1) Equal - The two styles are identical
+    // (2) Repaint - The object just needs to be repainted.
+    // (3) RepaintLayer - The layer and its descendant layers needs to be repainted.
+    // (4) Layout - A layout is required.
+    enum Diff { Equal, Repaint, RepaintLayer, Layout };
     Diff diff( const RenderStyle *other ) const;
 
+    bool isDisplayReplacedType() {
+        return display() == INLINE_BLOCK || display() == INLINE_BOX || display() == INLINE_TABLE;
+    }
     bool isDisplayInlineType() {
-        return display() == INLINE || display() == INLINE_BLOCK || display() == INLINE_BOX ||
-               display() == INLINE_TABLE;
+        return display() == INLINE || isDisplayReplacedType();
     }
     bool isOriginalDisplayInlineType() {
         return originalDisplay() == INLINE || originalDisplay() == INLINE_BLOCK ||
                originalDisplay() == INLINE_BOX || originalDisplay() == INLINE_TABLE;
     }
     
+    // To obtain at any time the pseudo state for a given link.
+    PseudoState pseudoState() const { return m_pseudoState; }
+    void setPseudoState(PseudoState s) { m_pseudoState = s; }
+    
+    // To tell if this style matched attribute selectors. This makes it impossible to share.
+    bool affectedByAttributeSelectors() const { return m_affectedByAttributeSelectors; }
+    void setAffectedByAttributeSelectors() { m_affectedByAttributeSelectors = true; }
+
     // Initial values for all the properties
     static bool initialBackgroundAttachment() { return true; }
     static EBackgroundRepeat initialBackgroundRepeat() { return REPEAT; }
@@ -1264,6 +1569,24 @@ public:
     static Length initialMarqueeIncrement() { return Length(6, Fixed); }
     static EMarqueeBehavior initialMarqueeBehavior() { return MSCROLL; }
     static EMarqueeDirection initialMarqueeDirection() { return MAUTO; }
+    static EUserModify initialUserModify() { return READ_ONLY; }
+    static EUserDrag initialUserDrag() { return DRAG_AUTO; }
+    static EUserSelect initialUserSelect() { return SELECT_AUTO; }
+    static bool initialTextOverflow() { return false; }
+    static EMarginCollapse initialMarginTopCollapse() { return MCOLLAPSE; }
+    static EMarginCollapse initialMarginBottomCollapse() { return MCOLLAPSE; }
+    static EWordWrap initialWordWrap() { return WBNORMAL; }
+    static ENBSPMode initialNBSPMode() { return NBNORMAL; }
+    static EKHTMLLineBreak initialKHTMLLineBreak() { return LBNORMAL; }
+    static EMatchNearestMailBlockquoteColor initialMatchNearestMailBlockquoteColor() { return BCNORMAL; }
+
+#if APPLE_CHANGES
+    // Keep these at the end.
+    static int initialLineClamp() { return -1; }
+    static bool initialTextSizeAdjust() { return true; }
+    static const QValueList<StyleDashboardRegion>& initialDashboardRegions();
+    static const QValueList<StyleDashboardRegion>& noneDashboardRegions();
+#endif
 };
 
 
