@@ -3,7 +3,7 @@
  *
  * (C) 1999 Lars Knoll (knoll@kde.org)
  * (C) 2000 Dirk Mueller (mueller@kde.org)
- * Copyright (C) 2003 Apple Computer, Inc.
+ * Copyright (C) 2004 Apple Computer, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,35 +32,80 @@
 
 #include <qptrvector.h>
 #include <assert.h>
+#include "KWQKHTMLPart.h"
 
 class QPainter;
 class QFontMetrics;
 
+namespace DOM {
+    class Position;
+    class DocumentMarker;
+};
+
+// Define a constant for soft hyphen's unicode value.
+#define SOFT_HYPHEN 173
+
+const int cNoTruncation = -1;
+const int cFullTruncation = -2;
+
 namespace khtml
 {
     class RenderText;
     class RenderStyle;
 
-class InlineTextBox : public InlineBox
+class InlineTextBox : public InlineRunBox
 {
 public:
     InlineTextBox(RenderObject* obj)
-    :InlineBox(obj)
+    :InlineRunBox(obj)
     {
         m_start = 0;
         m_len = 0;
         m_reversed = false;
+        m_treatAsText = true;
         m_toAdd = 0;
+        m_truncation = cNoTruncation;
     }
     
+    InlineTextBox* nextTextBox() const { return static_cast<InlineTextBox*>(nextLineBox()); }
+    InlineTextBox* prevTextBox() const { return static_cast<InlineTextBox*>(prevLineBox()); }
+    
+    uint start() const { return m_start; }
+    uint end() const { return m_len ? m_start+m_len-1 : m_start; }
+    uint len() const { return m_len; }
+
+    void offsetRun(int d) { m_start += d; }
+
     void detach(RenderArena* arena);
     
+    QRect selectionRect(int absx, int absy, int startPos, int endPos);
+    bool isSelected(int startPos, int endPos) const;
+    void selectionStartEnd(int& sPos, int& ePos);
+    
+    virtual void paint(RenderObject::PaintInfo& i, int tx, int ty);
+    virtual bool nodeAtPoint(RenderObject::NodeInfo& i, int x, int y, int tx, int ty);
+
+    RenderText* textObject();
+    
+    virtual void deleteLine(RenderArena* arena);
+    virtual void extractLine();
+    virtual void attachLine();
+
+    virtual RenderObject::SelectionState selectionState();
+
+    virtual void clearTruncation() { m_truncation = cNoTruncation; }
+    virtual int placeEllipsisBox(bool ltr, int blockEdge, int ellipsisWidth, bool& foundBox);
+
     // Overloaded new operator.  Derived classes must override operator new
     // in order to allocate out of the RenderArena.
     void* operator new(size_t sz, RenderArena* renderArena) throw();    
 
     // Overridden to prevent the normal delete from being called.
     void operator delete(void* ptr, size_t sz);
+    
+    // Current used only for sorting, == could be made more robust if needed
+    bool operator ==(const InlineTextBox &b) { return m_start == b.m_start; }
+    bool operator <(const InlineTextBox &b) { return m_start < b.m_start; }
         
 private:
     // The normal operator new is disallowed.
@@ -70,50 +115,43 @@ public:
     void setSpaceAdd(int add) { m_width -= m_toAdd; m_toAdd = add; m_width += m_toAdd; }
     int spaceAdd() { return m_toAdd; }
 
-    virtual bool isInlineTextBox() { return true; }
+    virtual bool isInlineTextBox() { return true; }    
+    virtual bool isText() const { return m_treatAsText; }
+    void setIsText(bool b) { m_treatAsText = b; }
+    
+    void paintDecoration(QPainter* p, int _tx, int _ty, int decoration);
+    void paintSelection(QPainter* p, int tx, int ty, RenderStyle* style, const Font* font);
+    void paintMarkedTextBackground(QPainter* p, int tx, int ty, RenderStyle* style, const Font* font, int startPos, int endPos);
+    void paintMarker(QPainter* p, int _tx, int _ty, DOM::DocumentMarker marker);
+    void paintMarkedTextUnderline(QPainter *pt, int _tx, int _ty, KWQKHTMLPart::MarkedTextUnderline underline);
+    virtual long caretMinOffset() const;
+    virtual long caretMaxOffset() const;
+    virtual unsigned long caretMaxRenderedOffset() const;
+    
+    int textPos() const;
+    int offsetForPosition(int _x, bool includePartialGlyphs = true);
     
-    void paintDecoration( QPainter *pt, int _tx, int _ty, int decoration);
-    void paintSelection(const Font *f, RenderText *text, QPainter *p, RenderStyle* style, int tx, int ty, int startPos, int endPos);
-
-    // Return before, after (offset set to max), or inside the text, at @p offset
-    FindSelectionResult checkSelectionPoint(int _x, int _y, int _tx, int _ty, const Font *f, RenderText *text, int & offset, short lineheight);
-
     /**
      * if this text run was rendered @ref _ty pixels below the upper edge
      * of a view, would the @ref _y -coordinate be inside the vertical range
      * of this object's representation?
      */
-    bool checkVerticalPoint(int _y, int _ty, int _h)
-    { if((_ty + m_y > _y + _h) || (_ty + m_y + m_baseline + height() < _y)) return false; return true; }
+    bool checkVerticalPoint(int _y, int _ty, int _h);
 
     int m_start;
     unsigned short m_len;
     
+    int m_truncation; // Where to truncate when text overflow is applied.  We use special constants to
+                      // denote no truncation (the whole run paints) and full truncation (nothing paints at all).
+
     bool m_reversed : 1;
+    bool m_treatAsText : 1; // Whether or not to treat a <br> as text for the purposes of line height.
     int m_toAdd : 14; // for justified text
+
 private:
-    // this is just for QVector::bsearch. Don't use it otherwise
-    InlineTextBox(int _x, int _y)
-        :InlineBox(0)
-    {
-        m_x = _x;
-        m_y = _y;
-        m_reversed = false;
-    };
     friend class RenderText;
 };
 
-class InlineTextBoxArray : public QPtrVector<InlineTextBox>
-{
-public:
-    InlineTextBoxArray();
-
-    InlineTextBox* first();
-
-    int	  findFirstMatching( Item ) const;
-    virtual int compareItems( Item, Item );
-};
-
 class RenderText : public RenderObject
 {
     friend class InlineTextBox;
@@ -128,39 +166,37 @@ public:
     virtual const char *renderName() const { return "RenderText"; }
 
     virtual void setStyle(RenderStyle *style);
-    
-    virtual void paint(QPainter *, int x, int y, int w, int h,
-                       int tx, int ty, PaintAction paintAction);
-    virtual void paintObject(QPainter *, int x, int y, int w, int h,
-                             int tx, int ty, PaintAction paintAction);
 
-    void deleteRuns();
+    void extractTextBox(InlineTextBox* textBox);
+    void attachTextBox(InlineTextBox* textBox);
+    void removeTextBox(InlineTextBox* textBox);
+    void deleteTextBoxes();
     virtual void detach();
     
     DOM::DOMString data() const { return str; }
     DOM::DOMStringImpl *string() const { return str; }
 
-    virtual InlineBox* createInlineBox(bool,bool);
+    virtual InlineBox* createInlineBox(bool,bool, bool isOnlyRun = false);
+    virtual void dirtyLineBoxes(bool fullLayout, bool isRootInlineBox = false);
     
-    virtual void layout() {assert(false);}
+    virtual void paint(PaintInfo& i, int tx, int ty) { assert(false); }
+    virtual void layout() { assert(false); }
 
     virtual bool nodeAtPoint(NodeInfo& info, int x, int y, int tx, int ty,
-                             HitTestAction hitTestAction = HitTestAll, bool inside=false);
+                             HitTestAction hitTestAction) { assert(false); return false; }
 
     virtual void absoluteRects(QValueList<QRect>& rects, int _tx, int _ty);
 
-    // Return before, after (offset set to max), or inside the text, at @p offset
-    virtual FindSelectionResult checkSelectionPointIgnoringContinuations
-        (int _x, int _y, int _tx, int _ty, DOM::NodeImpl*& node, int & offset);
+    virtual VisiblePosition positionForCoordinates(int x, int y);
 
     unsigned int length() const { return str->l; }
     QChar *text() const { return str->s; }
     unsigned int stringLength() const { return str->l; } // non virtual implementation of length()
     virtual void position(InlineBox* box, int from, int len, bool reverse);
 
-    virtual unsigned int width(unsigned int from, unsigned int len, const Font *f) const;
-    virtual unsigned int width(unsigned int from, unsigned int len, bool firstLine = false) const;
-    virtual short width() const;
+    virtual unsigned int width(unsigned int from, unsigned int len, const Font *f, int xpos) const;
+    virtual unsigned int width(unsigned int from, unsigned int len, int xpos, bool firstLine = false) const;
+    virtual int width() const;
     virtual int height() const;
 
     // height of the contents (without paddings, margins and borders)
@@ -169,13 +205,17 @@ public:
 
     // overrides
     virtual void calcMinMaxWidth();
-    virtual short minWidth() const { return m_minWidth; }
-    virtual short maxWidth() const { return m_maxWidth; }
-    virtual void trimmedMinMaxWidth(short& beginMinW, bool& beginWS, 
-                                    short& endMinW, bool& endWS,
+    virtual int minWidth() const { return m_minWidth; }
+    virtual int maxWidth() const { return m_maxWidth; }
+
+    // widths
+    void calcMinMaxWidth(int leadWidth);
+    virtual void trimmedMinMaxWidth(int leadWidth,
+                                    int& beginMinW, bool& beginWS, 
+                                    int& endMinW, bool& endWS,
                                     bool& hasBreakableChar, bool& hasBreak,
-                                    short& beginMaxW, short& endMaxW,
-                                    short& minW, short& maxW, bool& stripFrontSpaces);
+                                    int& beginMaxW, int& endMaxW,
+                                    int& minW, int& maxW, bool& stripFrontSpaces);
     
     bool containsOnlyWhitespace(unsigned int from, unsigned int len) const;
     
@@ -189,19 +229,18 @@ public:
     virtual const QFont &font();
     virtual short verticalPositionHint( bool firstLine ) const;
 
-    bool isFixedWidthFont() const;
-
     void setText(DOM::DOMStringImpl *text, bool force=false);
+    void setTextWithOffset(DOM::DOMStringImpl *text, uint offset, uint len, bool force=false);
 
+    virtual bool canBeSelectionLeaf() const { return true; }
     virtual SelectionState selectionState() const {return m_selectionState;}
-    virtual void setSelectionState(SelectionState s) {m_selectionState = s; }
-    virtual void cursorPos(int offset, int &_x, int &_y, int &height);
+    virtual void setSelectionState(SelectionState s);
+    virtual QRect selectionRect();
+    virtual QRect caretRect(int offset, EAffinity affinity, int *extraWidthToEndOfLine = 0);
     void posOfChar(int ch, int &x, int &y);
 
-    virtual short marginLeft() const { return style()->marginLeft().minWidth(0); }
-    virtual short marginRight() const { return style()->marginRight().minWidth(0); }
-
-    virtual int rightmostPosition() const;
+    virtual int marginLeft() const { return style()->marginLeft().minWidth(0); }
+    virtual int marginRight() const { return style()->marginRight().minWidth(0); }
 
     virtual QRect getAbsoluteRepaintRect();
 
@@ -211,36 +250,57 @@ public:
     DOM::TextImpl *element() const
     { return static_cast<DOM::TextImpl*>(RenderObject::element()); }
 
+    InlineTextBox* firstTextBox() const { return m_firstTextBox; }
+    InlineTextBox* lastTextBox() const { return m_lastTextBox; }
+    
+    virtual InlineBox *inlineBox(long offset, EAffinity affinity = UPSTREAM);
+
 #if APPLE_CHANGES
-    InlineTextBoxArray inlineTextBoxes() const { return m_lines; }
-    int widthFromCache(const Font *, int start, int len) const;
+    int widthFromCache(const Font *, int start, int len, int tabWidth, int xpos) const;
     bool shouldUseMonospaceCache(const Font *) const;
     void cacheWidths();
     bool allAscii() const;
 #endif
 
+    virtual long caretMinOffset() const;
+    virtual long caretMaxOffset() const;
+    virtual unsigned long caretMaxRenderedOffset() const;
+
+    virtual long previousOffset (long current) const;
+    virtual long nextOffset (long current) const;
+    
+    bool containsReversedText() { return m_containsReversedText; }
+    
 #if APPLE_CHANGES
 public:
 #endif
-    InlineTextBox * findNextInlineTextBox( int offset, int &pos );
+    InlineTextBox * findNextInlineTextBox( int offset, int &pos ) const;
 
 protected: // members
-    InlineTextBoxArray m_lines;
-    DOM::DOMStringImpl *str; //
-
-    short m_lineHeight;
-    short m_minWidth;
-    short m_maxWidth;
-    short m_beginMinWidth;
-    short m_endMinWidth;
+    DOM::DOMStringImpl *str;
+    
+    InlineTextBox* m_firstTextBox;
+    InlineTextBox* m_lastTextBox;
+    
+    int m_minWidth;
+    int m_maxWidth;
+    int m_beginMinWidth;
+    int m_endMinWidth;
     
     SelectionState m_selectionState : 3 ;
     bool m_hasBreakableChar : 1; // Whether or not we can be broken into multiple lines.
     bool m_hasBreak : 1; // Whether or not we have a hard break (e.g., <pre> with '\n').
+    bool m_hasTab : 1; // Whether or not we have a variable width tab character (e.g., <pre> with '\t').
     bool m_hasBeginWS : 1; // Whether or not we begin with WS (only true if we aren't pre)
     bool m_hasEndWS : 1; // Whether or not we end with WS (only true if we aren't pre)
     
-    // 19 bits left
+    bool m_linesDirty : 1; // This bit indicates that the text run has already dirtied specific
+                           // line boxes, and this hint will enable layoutInlineChildren to avoid
+                           // just dirtying everything when character data is modified (e.g., appended/inserted
+                           // or removed).
+    bool m_containsReversedText : 1;
+
+    // 22 bits left
 #if APPLE_CHANGES
     mutable bool m_allAsciiChecked:1;
     mutable bool m_allAscii:1;
