@@ -30,6 +30,7 @@
 #include "dom/dom_node.h"
 #include "dom/dom_misc.h"
 #include "misc/shared.h"
+#include "misc/main_thread_malloc.h"
 
 namespace DOM {
 
@@ -42,6 +43,8 @@ public:
     NodeFilterImpl(NodeFilterCondition *);
     ~NodeFilterImpl();
     
+    MAIN_THREAD_ALLOCATED;
+    
     short acceptNode(const Node &) const;
     
 private:
@@ -57,19 +60,13 @@ public:
     TraversalImpl(NodeImpl *, long whatToShow, NodeFilterImpl *, bool expandEntityReferences);
     ~TraversalImpl();
 
+    MAIN_THREAD_ALLOCATED;
+    
     NodeImpl *root() const { return m_root; }
     unsigned long whatToShow() const { return m_whatToShow; }
     NodeFilterImpl *filter() const { return m_filter; }
     bool expandEntityReferences() const { return m_expandEntityReferences; }
 
-    NodeImpl *findParentNode(NodeImpl *, short accept=NodeFilter::FILTER_ACCEPT) const;
-    NodeImpl *findFirstChild(NodeImpl *) const;
-    NodeImpl *findLastChild(NodeImpl *) const;
-    NodeImpl *findNextSibling(NodeImpl *) const;
-    NodeImpl *findPreviousSibling(NodeImpl *) const;
-    NodeImpl *findNextNode(NodeImpl *) const;
-    NodeImpl *findLastDescendant(NodeImpl *node) const;
-    NodeImpl *findPreviousNode(NodeImpl *) const;
     short acceptNode(NodeImpl *) const;
 
 private:
@@ -112,6 +109,8 @@ private:
     void setDetached(bool flag=true) { m_detached = flag; }
     DocumentImpl * document() const { return m_doc; }
     void setDocument(DocumentImpl *);
+    NodeImpl *findNextNode(NodeImpl *) const;
+    NodeImpl *findPreviousNode(NodeImpl *) const;
 
     NodeImpl *m_referenceNode;
     bool m_beforeReferenceNode;
@@ -143,6 +142,8 @@ private:
     // convenience for when it is known there will be no exception
     void setCurrentNode(NodeImpl *);
     
+    bool ancestorRejected(const NodeImpl *) const;
+    
     NodeImpl *m_current;
 };
 
