@@ -52,11 +52,13 @@ const float rightMargin = 2;
 
 @interface KWQTableView : NSTableView <KWQWidgetHolder>
 {
+@public
     QListBox *_box;
     BOOL processingMouseEvent;
     BOOL clickedDuringMouseEvent;
     BOOL inNextValidKeyView;
     NSWritingDirection _direction;
+    BOOL isSystemFont;
 }
 - (id)initWithListBox:(QListBox *)b;
 - (void)detach;
@@ -64,6 +66,7 @@ const float rightMargin = 2;
 - (QWidget *)widget;
 - (void)setBaseWritingDirection:(NSWritingDirection)direction;
 - (NSWritingDirection)baseWritingDirection;
+- (void)fontChanged;
 @end
 
 static id <WebCoreTextRenderer> itemScreenRenderer;
@@ -150,13 +153,13 @@ QListBox::QListBox(QWidget *parent)
     [scrollView setDocumentView:tableView];
     [tableView release];
     [scrollView setVerticalLineScroll:[tableView rowHeight]];
-    
+        
     KWQ_UNBLOCK_EXCEPTIONS;
 }
 
 QListBox::~QListBox()
 {
-    NSScrollView *scrollView = getView();
+    NSScrollView *scrollView = static_cast<NSScrollView *>(getView());
     
     KWQ_BLOCK_EXCEPTIONS;
     KWQTableView *tableView = [scrollView documentView];
@@ -172,7 +175,7 @@ void QListBox::clear()
 
 void QListBox::setSelectionMode(SelectionMode mode)
 {
-    NSScrollView *scrollView = getView();
+    NSScrollView *scrollView = static_cast<NSScrollView *>(getView());
 
     KWQ_BLOCK_EXCEPTIONS;
     NSTableView *tableView = [scrollView documentView];
@@ -180,9 +183,9 @@ void QListBox::setSelectionMode(SelectionMode mode)
     KWQ_UNBLOCK_EXCEPTIONS;
 }
 
-void QListBox::appendItem(const QString &text, bool isLabel)
+void QListBox::appendItem(const QString &text, KWQListBoxItemType type)
 {
-    _items.append(KWQListBoxItem(text, isLabel));
+    _items.append(KWQListBoxItem(text, type));
     _widthGood = false;
 }
 
@@ -190,7 +193,7 @@ void QListBox::doneAppendingItems()
 {
     KWQ_BLOCK_EXCEPTIONS;
 
-    NSScrollView *scrollView = getView();
+    NSScrollView *scrollView = static_cast<NSScrollView *>(getView());
     NSTableView *tableView = [scrollView documentView];
     [tableView reloadData];
 
@@ -203,7 +206,7 @@ void QListBox::setSelected(int index, bool selectIt)
 
     KWQ_BLOCK_EXCEPTIONS;
 
-    NSScrollView *scrollView = getView();
+    NSScrollView *scrollView = static_cast<NSScrollView *>(getView());
     NSTableView *tableView = [scrollView documentView];
     _changingSelection = true;
     if (selectIt) {
@@ -224,7 +227,7 @@ bool QListBox::isSelected(int index) const
 
     KWQ_BLOCK_EXCEPTIONS;
 
-    NSScrollView *scrollView = getView();
+    NSScrollView *scrollView = static_cast<NSScrollView *>(getView());
     NSTableView *tableView = [scrollView documentView];
     return [tableView isRowSelected:index]; 
 
@@ -236,7 +239,7 @@ bool QListBox::isSelected(int index) const
 void QListBox::setEnabled(bool enabled)
 {
     if (enabled != _enabled) {
-        // You would think this would work, but not until AppKit bug 2177792 if fixed.
+        // You would think this would work, but not until AppKit bug 2177792 is fixed.
         //KWQ_BLOCK_EXCEPTIONS;
         //NSTableView *tableView = [(NSScrollView *)getView() documentView];
         //[tableView setEnabled:enabled];
@@ -244,7 +247,7 @@ void QListBox::setEnabled(bool enabled)
 
         _enabled = enabled;
 
-        NSScrollView *scrollView = getView();
+        NSScrollView *scrollView = static_cast<NSScrollView *>(getView());
         NSTableView *tableView = [scrollView documentView];
         [tableView reloadData];
     }
@@ -261,7 +264,7 @@ QSize QListBox::sizeForNumberOfLines(int lines) const
 
     KWQ_BLOCK_EXCEPTIONS;
 
-    NSScrollView *scrollView = getView();
+    NSScrollView *scrollView = static_cast<NSScrollView *>(getView());
     KWQTableView *tableView = [scrollView documentView];
     
     if (!_widthGood) {
@@ -274,24 +277,48 @@ QSize QListBox::sizeForNumberOfLines(int lines) const
             style.rtl = [tableView baseWritingDirection] == NSWritingDirectionRightToLeft;
             style.applyRunRounding = NO;
             style.applyWordRounding = NO;
+            
+            id <WebCoreTextRenderer> renderer;
+            id <WebCoreTextRenderer> groupLabelRenderer;
+            
+            if (tableView->isSystemFont) {        
+                renderer = itemTextRenderer();
+                groupLabelRenderer = groupLabelTextRenderer();
+            }
+            else {
+                NSFont *f = font().getNSFont();
+                QFont b = font();
+                b.setWeight(QFont::Bold);
+                NSFont *boldFont = b.getNSFont();
+            
+            
+                renderer = [[WebCoreTextRendererFactory sharedFactory]
+                    rendererWithFont:f usingPrinterFont:![NSGraphicsContext currentContextDrawingToScreen]];
+
+                groupLabelRenderer = [[WebCoreTextRendererFactory sharedFactory]
+                    rendererWithFont:boldFont usingPrinterFont:![NSGraphicsContext currentContextDrawingToScreen]];
+            }
+            
             do {
                 const QString &s = (*i).string;
-                id <WebCoreTextRenderer> renderer = (*i).isGroupLabel ? groupLabelTextRenderer() : itemTextRenderer();
-                ++i;
 
                 WebCoreTextRun run;
                 int length = s.length();
                 WebCoreInitializeTextRun(&run, reinterpret_cast<const UniChar *>(s.unicode()), length, 0, length);
 
-                float textWidth = [renderer floatWidthForRun:&run style:&style widths:0];
+                float textWidth = [(((*i).type == KWQListBoxGroupLabel) ? groupLabelRenderer : renderer) floatWidthForRun:&run style:&style widths:0];
                 width = kMax(width, textWidth);
+                
+                ++i;
+            
             } while (i != e);
         }
         _width = ceilf(width);
         _widthGood = true;
     }
     
-    size = [NSScrollView frameSizeForContentSize:NSMakeSize(_width, [tableView rowHeight] * MAX(minLines, lines))
+    NSSize tempSize = { _width, [tableView rowHeight] * MAX(minLines, lines) }; // workaround for 4213314
+    size = [NSScrollView frameSizeForContentSize:tempSize
         hasHorizontalScroller:NO hasVerticalScroller:YES borderType:NSBezelBorder];
     size.width += [NSScroller scrollerWidthForControlSize:NSSmallControlSize] - [NSScroller scrollerWidth] + leftMargin + rightMargin;
 
@@ -323,7 +350,7 @@ void QListBox::setWritingDirection(QPainter::TextDirection d)
 {
     KWQ_BLOCK_EXCEPTIONS;
 
-    NSScrollView *scrollView = getView();
+    NSScrollView *scrollView = static_cast<NSScrollView *>(getView());
     KWQTableView *tableView = [scrollView documentView];
     NSWritingDirection direction = d == QPainter::RTL ? NSWritingDirectionRightToLeft : NSWritingDirectionLeftToRight;
     if ([tableView baseWritingDirection] != direction) {
@@ -349,6 +376,15 @@ void QListBox::clearCachedTextRenderers()
     groupLabelPrinterRenderer = nil;
 }
 
+void QListBox::setFont(const QFont &font)
+{
+    QWidget::setFont(font);
+
+    NSScrollView *scrollView = static_cast<NSScrollView *>(getView());
+    KWQTableView *tableView = [scrollView documentView];
+    [tableView fontChanged];
+}
+
 @implementation KWQListBoxScrollView
 
 - (id)initWithListBox:(QListBox *)b
@@ -403,7 +439,6 @@ void QListBox::clearCachedTextRenderers()
     [self setAllowsMultipleSelection:NO];
     [self setHeaderView:nil];
     [self setIntercellSpacing:NSMakeSize(0, 0)];
-    [self setRowHeight:ceilf([itemFont() ascender] - [itemFont() descender] + bottomMargin)];
     
     [self setDataSource:self];
     [self setDelegate:self];
@@ -563,7 +598,7 @@ void QListBox::clearCachedTextRenderers()
 
 - (BOOL)tableView:(NSTableView *)tableView shouldSelectRow:(int)row
 {
-    return _box && !_box->itemAtIndex(row).isGroupLabel;
+    return _box && _box->itemAtIndex(row).type == KWQListBoxOption;
 }
 
 - (BOOL)selectionShouldChangeInTableView:(NSTableView *)aTableView
@@ -592,8 +627,21 @@ void QListBox::clearCachedTextRenderers()
 
     bool RTL = _direction == NSWritingDirectionRightToLeft;
 
-    id <WebCoreTextRenderer> renderer = item.isGroupLabel ? groupLabelTextRenderer() : itemTextRenderer();
-
+    NSFont *font = _box->font().getNSFont();
+    
+    id <WebCoreTextRenderer> renderer;
+    if (isSystemFont) {
+        renderer = (item.type == KWQListBoxGroupLabel) ? groupLabelTextRenderer() : itemTextRenderer();
+    } else {
+        if (item.type == KWQListBoxGroupLabel) {
+            QFont boldFont = _box->font();
+            boldFont.setWeight(QFont::Bold);
+            font = boldFont.getNSFont();
+        }
+        renderer = [[WebCoreTextRendererFactory sharedFactory]
+            rendererWithFont:font usingPrinterFont:![NSGraphicsContext currentContextDrawingToScreen]];
+    }
+   
     WebCoreTextStyle style;
     WebCoreInitializeEmptyTextStyle(&style);
     style.rtl = RTL;
@@ -612,7 +660,7 @@ void QListBox::clearCachedTextRenderers()
     } else {
         point.x = NSMaxX(cellRect) - rightMargin - [renderer floatWidthForRun:&run style:&style widths:0];
     }
-    point.y = NSMaxY(cellRect) + [itemFont() descender] - bottomMargin;
+    point.y = NSMaxY(cellRect) + [font descender] - bottomMargin;
 
     WebCoreTextGeometry geometry;
     WebCoreInitializeEmptyTextGeometry(&geometry);
@@ -650,4 +698,13 @@ void QListBox::clearCachedTextRenderers()
     return cell;
 }
 
+- (void)fontChanged
+{
+    NSFont *font = _box->font().getNSFont();
+    isSystemFont = [[font fontName] isEqualToString:[itemFont() fontName]] && ([font pointSize] == [itemFont() pointSize]);
+    
+    [self setRowHeight:ceilf([font ascender] - [font descender] + bottomMargin)];
+    [self setNeedsDisplay:YES];
+}
+
 @end
