@@ -164,6 +164,14 @@ typedef enum {
     WebUndoActionTyping,
 } WebUndoAction;
 
+typedef enum
+{
+    ObjectElementNone,
+    ObjectElementImage,
+    ObjectElementFrame,
+    ObjectElementPlugin,
+} ObjectElementType;
+
 // WebCoreBridge objects are used by WebCore to abstract away operations that need
 // to be implemented by library clients, for example WebKit. The objects are also
 // used in the opposite direction, for simple access to WebCore functions without dealing
@@ -218,11 +226,14 @@ typedef enum {
 - (void)end;
 - (void)stop;
 
+- (void)handleFallbackContent;
+
 - (NSURL *)URL;
 - (NSURL *)baseURL;
 - (NSString *)referrer;
 - (NSString *)domain;
 - (WebCoreBridge *)opener;
+- (void)setOpener:(WebCoreBridge *)bridge;
 
 - (void)installInFrame:(NSView *)view;
 - (void)removeFromFrame;
@@ -461,6 +472,11 @@ typedef enum {
 - (WebCoreBridge *)createWindowWithURL:(NSURL *)URL frameName:(NSString *)name;
 - (void)showWindow;
 
+- (BOOL)canRunModal;
+- (BOOL)canRunModalNow;
+- (WebCoreBridge *)createModalDialogWithURL:(NSURL *)URL;
+- (void)runModal;
+
 - (NSString *)userAgentForURL:(NSURL *)URL;
 
 - (void)setTitle:(NSString *)title;
@@ -547,7 +563,7 @@ typedef enum {
 
 - (int)getObjectCacheSize;
 
-- (BOOL)frameRequiredForMIMEType:(NSString*)MIMEType URL:(NSURL *)URL;
+- (ObjectElementType)determineObjectFromMIMEType:(NSString*)MIMEType URL:(NSURL*)URL;
 
 - (void)loadEmptyDocumentSynchronously;
 
@@ -599,6 +615,7 @@ typedef enum {
 - (void)issueCopyCommand;
 - (void)issuePasteCommand;
 - (void)issuePasteAndMatchStyleCommand;
+- (void)issueTransposeCommand;
 - (void)respondToChangedSelection;
 - (void)respondToChangedContents;
 - (void)setIsSelected:(BOOL)isSelected forView:(NSView *)view;
