@@ -60,11 +60,12 @@
 #import "visible_units.h"
 #import "xml_tokenizer.h"
 
-#import <JavaScriptCore/npruntime.h>
+#import <JavaScriptCore/interpreter.h>
 #import <JavaScriptCore/jni_jsobject.h>
+#import <JavaScriptCore/npruntime.h>
 #import <JavaScriptCore/object.h>
-#import <JavaScriptCore/runtime_root.h>
 #import <JavaScriptCore/property_map.h>
+#import <JavaScriptCore/runtime_root.h>
 
 #import "KWQAssertions.h"
 #import "KWQCharsets.h"
@@ -136,6 +137,8 @@ using khtml::UPSTREAM;
 using khtml::VisiblePosition;
 
 using KJS::ExecState;
+using KJS::Interpreter;
+using KJS::InterpreterLock;
 using KJS::ObjectImp;
 using KJS::SavedProperties;
 using KJS::SavedBuiltins;
@@ -201,7 +204,7 @@ static BOOL partHasSelection(WebCoreBridge *bridge)
     if (!bridge)
         return NO;
     
-    KHTMLPart *part = bridge->_part;
+    KHTMLPart *part = [bridge part];
     if (!part)
         return NO;
         
@@ -467,6 +470,8 @@ static bool initializedKJS = FALSE;
     }
     _part->clearTimers();
 
+    InterpreterLock lock;
+
     SavedProperties *windowProperties = new SavedProperties;
     _part->saveWindowProperties(windowProperties);
 
@@ -506,6 +511,11 @@ static bool initializedKJS = FALSE;
     _part->stop();
 }
 
+- (void)handleFallbackContent
+{
+    _part->handleFallbackContent();
+}
+
 - (void)createKHTMLViewWithNSView:(NSView *)view marginWidth:(int)mw marginHeight:(int)mh
 {
     // If we own the view, delete the old one - otherwise the render _part will take care of deleting the view.
@@ -1083,7 +1093,7 @@ static HTMLFormElementImpl *formElementFromDOMElement(DOMElement *element)
     
         if (node->renderer() && node->renderer()->isImage()) {
             RenderImage *r = static_cast<RenderImage *>(node->renderer());
-            NSImage *image = r->pixmap().image();
+            NSImage * image = (NSImage *)(r->pixmap().image());
             // Only return image information if there is an image.
             if (image && !r->isDisplayingError()) {
                 [element setObject:r->pixmap().image() forKey:WebCoreElementImageKey];
@@ -1350,6 +1360,14 @@ static HTMLFormElementImpl *formElementFromDOMElement(DOMElement *element)
     return nil;
 }
 
+- (void)setOpener:(WebCoreBridge *)bridge;
+{
+    KHTMLPart *p = [bridge part];
+    
+    if (p)
+        p->setOpener(_part);
+}
+
 + (NSString *)stringWithData:(NSData *)data textEncoding:(CFStringEncoding)textEncoding
 {
     if (textEncoding == kCFStringEncodingInvalidId || textEncoding == kCFStringEncodingISOLatin1) {
@@ -2268,7 +2286,7 @@ static HTMLFormElementImpl *formElementFromDOMElement(DOMElement *element)
     VisiblePosition caret(selection.start(), selection.startAffinity());
     VisiblePosition next = caret.next();
     VisiblePosition previous = caret.previous();
-    if (caret == next || caret == previous)
+    if (previous.isNull() || next.isNull() || caret == next || caret == previous)
         return nil;
 
     return [DOMRange _rangeWithImpl:makeRange(previous, next).handle()];
